/*
 * Pan - A Newsreader for X
 * Copyright (C) 1999  Pan Development Team (pan@superpimp.org)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 */

#ifndef __ARTICLE_H__
#define __ARTICLE_H__

#include <time.h>
#include <glib.h>
#include "group.h"
#include "pan-callback.h"
#include "pan-object.h"

#define ARTICLE_DATA(a) ((article_data *)a)

enum {
	STATE_NONE		= 0,
	STATE_MULTIPART_ALL	= 1<<0,
	STATE_MULTIPART_SOME	= 1<<1,
	STATE_DOWNLOAD_QUEUED	= 1<<2,
	STATE_DECODE_QUEUED	= 1<<3,
	STATE_READ		= 1<<4,
	STATE_DECODED		= 1<<5,
	STATE_ERROR		= 1<<6,
	STATE_NEW		= 1<<7,
	STATE_CACHED		= 1<<8,
	STATE_DECODE_FAILED	= 1<<9,
	STATE_KILLFILE		= 1<<10, /* only used in articlelist */
	STATE_WATCHED		= 1<<11 /* only used in articlelist */
};

typedef struct _article_data
{
        /* parent class */
	PanObject pan_object;

	/* basic info */
	gchar *number;
	gchar *subject;
	gchar *author;
	gchar *message_id;
	gchar *references;
	gint linecount;
	time_t date;

	/* optional info */
	gchar *reply_to;
	gchar *followup_to;
	gchar *newsgroups;

	/* multipart info */
	gint parts;			/* no. of parts to whole multipart message */
	gint part;			/* current part no. */

	/* group */
	gulong state;

	/* articlelist */
	struct _article_data* parent;	/* parent in threaded view */
	GSList *threads;		/* linked list of child threads */
	gint unread_children;
	gboolean self_passes_filter;
	gboolean tree_passes_filter;
}
article_data;

extern PanCallback* article_deleted;

extern void article_init (void);

/**
 * This reparents all of adata's children to be children of adata's parent,
 * removes adata from its parent's child list, and
 * clears out all of adata's parent/child references.
 */
void article_isolate (article_data*);

article_data* article_new (void);

gchar* article_get_thread_message_id (const article_data*);

/**
***  PROTECTED
**/

void article_constructor (article_data*);

void article_destructor (PanObject*);

/**
***  PUBLIC
**/

#define article_flag_on(a, flag) (a && (a->state&flag))

void article_mark_read (const server_data*,
                        const group_data*,
                        article_data*,
                        gboolean sync);

void articles_mark_read (const server_data*,
                         const group_data*,
                         article_data**,
                         int article_qty,
                         gboolean sync);

void article_mark_unread (const server_data*,
                          const group_data*,
                          article_data*,
                          gboolean sync);

void articles_mark_unread (const server_data*,
                           const group_data*,
                           article_data** mark_us,
                           int article_qty,
                           gboolean sync);

void article_delete (const server_data*,
                     const group_data*,
                     article_data*,
                     gboolean sync);

void articles_delete (const server_data*,
                      const group_data*,
                      article_data**,
                      int article_qty,
                      gboolean sync);

/**
***
**/

void article_add_flag (const server_data*,
                       const group_data*,
                       article_data*,
                       int flag,
                       gboolean sync);

void article_remove_flag (const server_data*,
                          const group_data*,
                          article_data*,
                          int flag,
                          gboolean sync);

void article_update (const server_data*,
                     const group_data*,
                     const article_data*,
                     gboolean sync);

/**
***
**/

/**
 * @param adata the article whose thread is to be returned
 * @return a bag of all the articles threaded into this article.
 *         adata's parent/children fields are traversed for this.
 */
void article_get_entire_thread (article_data     *adata,
                                int              *setme_qty,
                                article_data  ***setme_buf);

void article_copy_to_folder (article_data *adata,
                             group_data   *from,
                             group_data   *to);

#endif /* __ARTICLE_H__ */
